import java.util.ArrayList;
import java.util.List;

import com.cryptware.jcryptoki.Cryptoki;
import com.cryptware.jcryptoki.CryptokiAttribute;
import com.cryptware.jcryptoki.CryptokiException;
import com.cryptware.jcryptoki.CryptokiInfo;
import com.cryptware.jcryptoki.CryptokiObject;
import com.cryptware.jcryptoki.Key;
import com.cryptware.jcryptoki.Mechanism;
import com.cryptware.jcryptoki.PrivateKey;
import com.cryptware.jcryptoki.RSAPrivateKey;
import com.cryptware.jcryptoki.RSAPublicKey;
import com.cryptware.jcryptoki.Session;
import com.cryptware.jcryptoki.SessionInfo;
import com.cryptware.jcryptoki.Slot;
import com.cryptware.jcryptoki.SlotInfo;
import com.cryptware.jcryptoki.SlotList;
import com.cryptware.jcryptoki.Token;
import com.cryptware.jcryptoki.TokenInfo;
import com.cryptware.jcryptoki.X509Certificate;


public class TestJCryptoki 
{
	public static void main(String[] args)
	{
		Cryptoki cryptoki = new Cryptoki();
		
		cryptoki.attach("smaoscki");

		int ret = cryptoki.initialize(true);
		if(ret != CryptokiException.CKR_OK)
		{
			System.out.println("Error " + ret);
			return;
		}
		
		CryptokiInfo info = cryptoki.getInfo();
		
		System.out.println(info.getLibDescription());
		System.out.println(info.getLibVersion());
		System.out.println(info.getVersion());
		System.out.println(info.getManufacturerID());
		
		// Reads the set of slots containing a token
        SlotList slots = cryptoki.getActiveSlots();
        Slot slot;
        if (slots.size() == 0)
        {
            slot = cryptoki.waitForSlotEvent(0);
        }
        else
        {
        	slot = (Slot) slots.get(0);
        }
        
		
		SlotInfo slotInfo = slot.getInfo();
		
		System.out.println(slotInfo.getDescription());
		System.out.println(slotInfo.getFirmwareVersion());
		System.out.println(slotInfo.getHardwareVersion());
		System.out.println(slotInfo.getManufacturerID());

		if(slot.isTokenPresent())
		{
			Token token = slot.getToken();
			
			TokenInfo tokenInfo = token.getInfo();

			System.out.println(tokenInfo.getLabel());
			System.out.println(tokenInfo.getFirmwareVersion());
			System.out.println(tokenInfo.getHardwareVersion());
			System.out.println(tokenInfo.getManufacturerID());
		
	        // Opens a read/write serial session
	        Session session = token.openSession(SessionInfo.CKF_SERIAL_SESSION | SessionInfo.CKF_RW_SESSION,
	                                     null,
	                                     null);

	        SessionInfo sessionInfo = session.getInfo();
	        
	        System.out.println(sessionInfo.getSlotID());
			System.out.println(sessionInfo.getState());
			
	        // Executes the login passing the user PIN
	        int nRes = session.login(Session.CKU_USER, "12345678");
	        if (nRes != 0)
	        {
	            System.out.println("Wrong PIN");
	            return;
	        }

	        // Searchs for an RSA private key object
	        // Sets the template with its attributes
	        ArrayList<CryptokiAttribute> template = new ArrayList<CryptokiAttribute>();
	        
	        template.add(new CryptokiAttribute(CryptokiAttribute.CKA_CLASS, CryptokiObject.CKO_PRIVATE_KEY));
	        template.add(new CryptokiAttribute(CryptokiAttribute.CKA_KEY_TYPE, Key.CKK_RSA));
	        template.add(new CryptokiAttribute(CryptokiAttribute.CKA_LABEL, "Ugo's new Key"));

	        // Launchs the search specifying the template just created
	        List<CryptokiObject> objects = session.getObjects().find(template, 10);

	        for (CryptokiObject obj : objects)
	        {
	            System.out.println(((PrivateKey)obj).getLabel());
	            System.out.println(((PrivateKey)obj).getID());
	        }

	        RSAPrivateKey privateKey;
	        RSAPublicKey publicKey;

	        // If the private keys is found computes the digital signature 
	        if (objects.size() == 0)
	        {
	        	ArrayList<CryptokiAttribute> templatePub = new ArrayList<CryptokiAttribute>();
	            templatePub.add(new CryptokiAttribute(CryptokiAttribute.CKA_CLASS, CryptokiObject.CKO_PUBLIC_KEY));
	            templatePub.add(new CryptokiAttribute(CryptokiAttribute.CKA_TOKEN, true));
	            templatePub.add(new CryptokiAttribute(CryptokiAttribute.CKA_PRIVATE, false));
	            templatePub.add(new CryptokiAttribute(CryptokiAttribute.CKA_LABEL, "Ugo's new Key"));
	            templatePub.add(new CryptokiAttribute(CryptokiAttribute.CKA_ID, "1"));
	            templatePub.add(new CryptokiAttribute(CryptokiAttribute.CKA_MODULUS_BITS, 1024));
	            templatePub.add(new CryptokiAttribute(CryptokiAttribute.CKA_PUBLIC_EXPONENT, 0x010001));
	
	
	            ArrayList<CryptokiAttribute> templatePri = new ArrayList<CryptokiAttribute>();
	            templatePri.add(new CryptokiAttribute(CryptokiAttribute.CKA_CLASS, CryptokiObject.CKO_PRIVATE_KEY));
	            templatePri.add(new CryptokiAttribute(CryptokiAttribute.CKA_TOKEN, true));
	            templatePri.add(new CryptokiAttribute(CryptokiAttribute.CKA_PRIVATE, true));
	            templatePri.add(new CryptokiAttribute(CryptokiAttribute.CKA_LABEL, "Ugo's new Key"));
	            templatePri.add(new CryptokiAttribute(CryptokiAttribute.CKA_ID, "1".getBytes()));
	
	
	            //gets the first object
	            Key[] keys = session.generateKeyPair(new Mechanism(Mechanism.CKM_RSA_PKCS_KEY_PAIR_GEN, null), templatePub, templatePri);
	            privateKey = (RSAPrivateKey)keys[1];
	            publicKey = (RSAPublicKey)keys[0];	            
	        }
	        else
	        {
	            privateKey = (RSAPrivateKey)objects.get(objects.size() - 1);
	            System.out.println(privateKey.getLabel());
	
	            template = new ArrayList<CryptokiAttribute>();
		        
		        template.add(new CryptokiAttribute(CryptokiAttribute.CKA_CLASS, CryptokiObject.CKO_PUBLIC_KEY));
		        template.add(new CryptokiAttribute(CryptokiAttribute.CKA_KEY_TYPE, Key.CKK_RSA));
		        template.add(new CryptokiAttribute(CryptokiAttribute.CKA_LABEL, "Ugo's new Key"));

		        // Launchs the search specifying the template just created
		        List<CryptokiObject> pubKeys = session.getObjects().find(template, 10);

		        publicKey = (RSAPublicKey)pubKeys.get(0);
		        
	            template = new ArrayList<CryptokiAttribute>();
	            template.add(new CryptokiAttribute(CryptokiAttribute.CKA_CLASS, CryptokiObject.CKO_CERTIFICATE));
	
	            // Launchs the search specifying the template just created  
	            objects = session.getObjects().find(template, 10);
	            if (objects.size() < 1)
	            {
	            	System.out.println("Certificate not found");	              
	            }
	
	            for (CryptokiObject obj : objects)
	            {
	            	System.out.println(((X509Certificate)obj).getLabel());
	            	System.out.println(new String(((X509Certificate)obj).getID()));
	            	System.out.println(new String(((X509Certificate)obj).getSubject()));
	            }
	
	            //X509Certificate cert = (X509Certificate)objects.get(0);

	        }	
	        
	        // launches the digital signing operation with a RSA_PKCS mechanism
            nRes = session.signInit(new Mechanism(Mechanism.CKM_SHA1_RSA_PKCS, null), privateKey);

            // computes the signature
            byte[] signature = session.sign("ciao ciao".getBytes());

            nRes = session.verifyInit(new Mechanism(Mechanism.CKM_SHA1_RSA_PKCS, null), publicKey);

            nRes = session.verify("ciao ciao".getBytes(), signature);
            
            System.out.println(nRes);
        
            session.logout();
            
            session.close();
		}
		
		cryptoki.finalize(null);
		
		cryptoki.detach();
	}
}
